import termcolor


class Conference:
    def __init__(self, name, venue, short, keywords) -> None:
        self.name = name
        self.venue = venue
        self.short = short
        self.keywords = keywords

    def __str__(self):
        if len(self.keywords) > 0:
            ccf_class = self.keywords[0]
            if ccf_class.endswith("A"):
                return "{} - {} ({})".format(termcolor.colored(self.keywords[0], "green"), self.name, self.short)
            elif ccf_class.endswith("B"):
                return "{} - {} ({})".format(termcolor.colored(self.keywords[0], "yellow"), self.name, self.short)
            elif ccf_class.endswith("C"):
                return "{} - {} ({})".format(termcolor.colored(self.keywords[0], "magenta"), self.name, self.short)
            else:
                return "{} - {} ({})".format(self.keywords[0], self.name, self.short)
        else:
            return "{} ({})".format(self.name, self.short)


Conferences = set()

# Cyber Security CCF A
Conferences_CCF_A = set()
Conferences_CCF_A.add(Conference(name="ACM Conferenceon Computer and Communications Security",
                              venue="CCS", short="CCS", keywords=["CCF - A"]))
Conferences_CCF_A.add(Conference(name="International Cryptology Conference",
                              venue="CRYPTO", short="CRYPTO", keywords=["CCF - A"]))
Conferences_CCF_A.add(Conference(name="European Cryptology Conference",
                              venue="EUROCRYPT", short="EUROCRYPT", keywords=["CCF - A"]))
Conferences_CCF_A.add(Conference(name="IEEE Symposium on Security and Privacy",
                              venue="IEEE_Symposium_on_Security_and_Privacy_Workshops", short="S&P", keywords=["CCF - A"]))
Conferences_CCF_A.add(Conference(name="Usenix Security Symposium",
                              venue="USENIX_Security_Symposium", short="USENIX", keywords=["CCF - A"]))

# Cyber Security CCF B
Conferences_CCF_B = set()
Conferences_CCF_B.add(Conference(name="Annual Computer Security Applications Conference",
                              venue="ACSAC", short="ACSAC", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="Annual International Conference on the Theory and Application of Cryptology and Information Security",
                              venue="ASIACRYPT", short="ASIACRYPT", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="Conference on Cryptographic Hardware and Embedded Systems",
                              venue="CHES", short="CHES", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="IEEE Computer Security Foundations Symposium",
                              venue="CSF", short="CSF", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="International Conference on Dependable Systems and Networks",
                              venue="DSN", short="DSN", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="European Symposiumon Research in Computer Security",
                              venue="ESORICS", short="ESORICS", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="Fast Software Encryption",
                              venue="FSE", short="FSE", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="Network and Distributed System Security Symposium",
                              venue="NDSS", short="NDSS", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="International Workshop on Practice and Theory in Public Key Cryptography",
                              venue="PKC", short="PKC", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="International Symposium on Research in Attacks, Intrusions, and Defenses",
                              venue="RAID", short="RAID", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="IEEE International Symposium on Reliable Distributed Systems",
                              venue="SRDS", short="SRDS", keywords=["CCF - B"]))
Conferences_CCF_B.add(Conference(name="Theory of Cryptography Conference",
                              venue="TCC", short="TCC", keywords=["CCF - B"]))

# Cyber Security CCF C
Conferences_CCF_C = set()
Conferences_CCF_C.add(Conference(name="Australasia Conference on Information Security and Privacy",
                              venue="ACISP", short="ACISP", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="International Conference on Applied Cryptography and Network Security",
                              venue="ACNS", short="ACNS", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="ACM Asia Conference on Computer and Communications Security",
                              venue="ASIACCS", short="ASIACCS", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="EAI International Conference on Cyber Attacks Response and Defense",
                              venue="CARDS", short="CARDS", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="RSA Conference, Cryptographers' Track",
                              venue="CT-RSA", short="RSA", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Digital Forensic Research Workshop",
                              venue="DFRWS-EU", short="EU", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Digital Forensic Research Workshop",
                              venue="DFRWS-USA", short="USA", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Detection of Intrusions and Malware & Vulnerability Assessment",
                              venue="DIMVA", short="DIMVA", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Financial Cryptography and Data Security",
                              venue="FC", short="FC", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="USENIX Workshop on Hot Topics in Security",
                              venue="HotSec", short="HotSec", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="International Conference on Information and Communications Security",
                              venue="ICICS", short="ICICS", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="IFIP WG 11.9 International Conference on Digital Forensics",
                              venue="IFIP WG 11.9", short="9", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="ACM Workshop on Information Hiding and Multimedia Security",
                              venue="IH&MMSec", short="MMSec", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Information Security Conference",
                              venue="ISC", short="ISC", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="New Security Paradigms Workshop",
                              venue="NSPW", short="NSPW", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Passive and Active Measurement Conference",
                              venue="PAM", short="PAM", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Privacy Enhancing Technologies Symposium",
                              venue="PETS", short="PETS", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Selected Areas in Cryptography",
                              venue="SAC", short="SAC", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="ACM Symposium on Access Control Models and Technologies",
                              venue="SACMAT", short="SACMAT", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="IFIP International Information Security and Privacy Conference",
                              venue="SEC", short="SEC", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="Symposium On Usable Privacy and Security",
                              venue="SOUPS", short="SOUPS", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="International Conference on Security and Privacy in Communication Networks",
                              venue="SecureComm", short="SecureComm", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="IEEE International Conference on Trust, Security and Privacy in Computing and Communications",
                              venue="TrustCom", short="TrustCom", keywords=["CCF - C"]))
Conferences_CCF_C.add(Conference(name="ACM Conference on Security and Privacy in Wireless and Mobile Networks",
                              venue="WISEc", short="WISEc", keywords=["CCF - C"]))

# Network CCF A
Conferences_CCF_A.add(Conference(name="IEEE International Conference on Computer Communications",
                              venue="INFOCOM", short="INFOCOM", keywords=["CCF - A"]))
Conferences_CCF_A.add(Conference(name="ACM Special Interest Group on Data Communication",
                              venue="SIGCOMM", short="SIGCOMM", keywords=["CCF - A"]))

# Not in CCF Recommendations
Conferences.add(Conference(
    name="Computing Research Repository", venue="CoRR", short="CoRR", keywords=[]))
Conferences.add(Conference(name="IEEE Access",
                              venue="IEEE_Access", short="ACCESS", keywords=[]))
Conferences.add(Conference(name="IEEE European Symposium on Security and Privacy",
                              venue="EuroS&P", short="EuroS&P", keywords=["CCF - A"]))
Conferences.add(Conference(
    name="IEEE International Conference on Communications", venue="ICC", short="ICC", keywords=[]))

Conferences = Conferences.union(Conferences_CCF_A)
Conferences = Conferences.union(Conferences_CCF_B)
Conferences = Conferences.union(Conferences_CCF_C)


import os
import requests
import termcolor
import glob
import argparse

from .core.article import Article
from .core.conference import Conferences
from .core.conference import Conferences_CCF_A
from .core.conference import Conferences_CCF_B
from .core.conference import Conferences_CCF_C
from bs4 import BeautifulSoup

__version__ = "0.0.5"


def print_success(x): return termcolor.cprint(x, 'green')
def print_error(x): return termcolor.cprint(x, 'red')
def print_warning(x): return termcolor.cprint(x, 'yellow')
def print_info(x): return termcolor.cprint(x)


def legalise_filename(filename):
    illegal_chars = {
        "|": "OR",
        "/": "-",
        "(": "[",
        ")": "]",
    }
    legal_filename = filename
    for k, v in illegal_chars.items():
        legal_filename = legal_filename.replace(k, v)
    return legal_filename


def legalise_area(area):
    result = area.strip()
    eliminations = [
        "(", ")", "|", " ",
    ]
    for elimination in eliminations:
        keyword = (" {}".format(elimination))
        while keyword in result:
            result = result[:result.index(
                keyword)] + elimination + result[result.index(keyword) + len(keyword):]
        keyword = ("{} ".format(elimination))
        while keyword in result:
            result = result[:result.index(
                keyword)] + elimination + result[result.index(keyword) + len(keyword):]
    for elimination in eliminations[0:-1]:
        result = result.replace(elimination, " {} ".format(elimination))
    return result.strip().replace("  ", " ")


class DBLP:
    def search(self, area, conf):
        if conf:
            keyword = "{} venue:{}:".format(area, conf.venue)
        else:
            keyword = area
        print_info(
            "[*] Searching {} on {}".format(termcolor.colored(area, "cyan"), conf))
        params = {
            "q": keyword,
            "h": 100,
            "f": 0,
            "s": "ydvspc",
        }
        url = "http://dblp.uni-trier.de/search/publ/inc"

        response = requests.get(url, params=params)
        content = response.text
        articles = self.parse(content)
        return articles

    def save(self, base_folder, area, conf, articles):
        folder = os.path.join(base_folder, legalise_filename(
            area), legalise_filename(conf.short))
        if len(articles) > 0:
            try:
                os.makedirs(folder)
            except:
                pass
        for article in articles:
            print("\t{}".format(termcolor.colored(article, "green")), end="")
            filename = legalise_filename("{}.bib".format(article.dblp_key))
            filepath = os.path.join(folder, filename)
            if not os.path.exists(filepath):
                with open(filepath, "w") as f:
                    f.write(article.get_bibtex())
            print(" {} {}".format(termcolor.colored("=>", "yellow"),
                                  termcolor.colored(filepath, "cyan")))

    def parse(self, content):
        soup = BeautifulSoup(content, 'html.parser')
        articles_li = soup.find_all(class_="toc")
        articles = []
        for article_li in articles_li:
            doi_link = article_li.find("a")["href"]
            title = article_li.find("span", class_="title").text
            author_spans = article_li.find_all("span", itemprop="author")
            authors = []
            for author_span in author_spans:
                authors.append(author_span.find("span")["title"])
            confname = article_li.find("span", itemprop="isPartOf").find(
                "span", itemprop="name").text
            pub_year = article_li.find("span", itemprop="datePublished").text
            dblp_key = article_li.find(
                "ul", class_="bullets").find("small").text
            bibtex_link = article_li.find("nav", class_="publ").find_all(
                "li", class_="drop-down")[1].find("div", class_="head").find("a")["href"]
            article = Article(title, authors, confname,
                              pub_year, dblp_key, doi_link, bibtex_link)
            articles.append(article)
        return articles


def summary(base_folder, area):
    # Save summary bibtex
    folder = os.path.join(base_folder, legalise_filename(area))
    bibtexes = glob.glob(os.path.join(folder, "**", "*.bib"))
    bibtexes_contents = []
    for bibtex in bibtexes:
        with open(bibtex, "r") as f:
            bibtexes_contents.append(f.read())
    if len(bibtexes_contents) > 0:
        with open(os.path.join(folder, "summary.bib"), "w") as f:
            f.write("\n".join(bibtexes_contents))


def main():

    parser = argparse.ArgumentParser(description='DBLP BibTeX Spider')
    parser.add_argument('--keywords', required=True,
                        nargs='+', help='DBLP searching keywords')

    parser.add_argument('--ccf-a', required=False,
                        help='Search in CCF A recommandation venues', action="store_true")
    parser.add_argument('--ccf-b', required=False,
                        help='Search in CCF B recommandation venues', action="store_true")
    parser.add_argument('--ccf-c', required=False,
                        help='Search in CCF C recommandation venues', action="store_true")

    parser.add_argument('--output', required=True, help='Output BibTeX folder')
    parser.add_argument('--version', action='version', version=__version__)
    args = parser.parse_args()

    dblp = DBLP()

    venues = set()
    if (not args.ccf_a) and (not args.ccf_b) and (not args.ccf_c):
        venues = venues.union(Conferences)
    else:
        if args.ccf_a:
            venues = venues.union(Conferences_CCF_A)
        if args.ccf_b:
            venues = venues.union(Conferences_CCF_B)
        if args.ccf_c:
            venues = venues.union(Conferences_CCF_C)

    for area in args.keywords:
        for venue in venues:
            legal_area = legalise_area(area)
            articles = dblp.search(legal_area, venue)
            if len(articles) == 0:
                print_error("[{}] articles found".format(len(articles)))
            else:
                print_info("[{}] articles found".format(len(articles)))
            dblp.save(args.output, legal_area, venue, articles)
        summary(args.output, legal_area)


if __name__ == "__main__":
    main()



import requests
from bs4 import BeautifulSoup


class Article:
    def __init__(self, title, authors, conf, year, dblp_key, doi_link, bibtex_link) -> None:
        self.title = title
        self.authors = authors
        self.doi_link = doi_link
        self.conf = conf
        self.year = year
        self.dblp_key = dblp_key
        self.bibtex_link = bibtex_link

    def __str__(self):
        if len(self.authors) > 0:
            return "[{} - {}] {} - {}".format(self.conf, self.year, self.title, self.authors[0])
        else:
            return "[{} - {}] {}".format(self.conf, self.year, self.title)

    def get_bibtex(self):
        response = requests.get(self.bibtex_link)
        content = response.text
        soup = BeautifulSoup(content, 'html.parser')
        bibtex = soup.find("pre", class_="verbatim select-on-click").text
        return bibtex



import setuptools
import dblp_spider

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="DBLP-Spider",
    version=dblp_spider.__version__,
    author="Wang Yihang",
    author_email="wangyihanger@gmail.com",
    description="A spider tool for downloading the DBLP search results into local BibTeX files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/WangYihang/Platypus-Python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.6",
    keywords="academic, research, spider, dblp, ccf",
    install_requires=["termcolor", "requests", "bs4"],
    entry_points={
        "console_scripts": [
            "dblp-spider=dblp_spider:main",
        ],
    },
)
